@@ -1,3 +1,3 @@
 def task_checker():
     return {'actions': ["pychecker sample.py"],
-            'dependencies': ["sample.py"]}
+            'file_dep': ["sample.py"]}
