@@ -11,6 +11,9 @@ import anydbm as ddbm
 #import dbm as ddbm # test_corrupted_file fails
 #import gdbm as ddbm # ok <= TODO make this the default
 
+# note: to check which DBM backend is being used:
+#       >>> anydbm._defaultmod
+
 from .compat import json
 
 
@@ -180,10 +183,14 @@ class DbmDB(object):
         self.dirty.add(task_id)
 
     def _in_dbm(self, key):
-        """python3: when for get/set key is convert to bytes but not for 'in'"""
-        # should be just
-        # return key in self._dbm
-        return key.encode('utf-8') in self._dbm
+        """
+        should be just::
+          return key in self._dbm
+
+         python3: when for get/set key is convert to bytes but not for 'in'
+         fix #23 'in' operator not supported on DBM implementations on py2.6
+        """
+        return self._dbm.has_key(key.encode('utf-8'))
 
 
     def get(self, task_id, dependency):
@@ -224,6 +231,9 @@ class DbmDB(object):
         if isinstance(self._dbm, dumbdbm._Database): # pragma: no cover
             self._dbm._index = {}
             self._dbm.close()
+        # gdbm can not be running on 2 instances on same thread
+        # see https://bitbucket.org/schettino72/doit/issue/16/
+        del self._dbm
         self._dbm = ddbm.open(self.name, 'n')
         self.dirty = set()
 
@@ -286,12 +296,6 @@ class DependencyBase(object):
             size = os.path.getsize(dep)
             self._set(task.name, dep, (timestamp, size, md5sum(dep)))
 
-        # result-dep
-        for dep in task.result_dep:
-            result = self._get(dep, "result:")
-            if result is not None:
-                self._set(task.name, "task:" + dep, result)
-
 
     def get_values(self, task_name):
         """get all saved values from a task
@@ -324,10 +328,11 @@ class DependencyBase(object):
 
 
     # TODO add option to log this
-    def get_status(self, task):
+    def get_status(self, task, tasks_dict):
         """Check if task is up to date. set task.dep_changed
 
         @param task: (Task)
+        @param tasks_dict: (dict: Task) passed to objects used on uptodate
         @return: (str) one of up-to-date, ignore, run
 
         task.dep_changed (list-strings): file-dependencies that are not
@@ -343,10 +348,17 @@ class DependencyBase(object):
         # check uptodate bool/callables
         checked_uptodate = False
         for utd, utd_args, utd_kwargs in task.uptodate:
-            # FIXME control verbosity, check error messages
+            # if parameter is a callable
             if hasattr(utd, '__call__'):
+                # FIXME control verbosity, check error messages
+                # 1) prepare arguments
                 args = [task, self.get_values(task.name)] + utd_args
+                # 2) setup object with global info all tasks
+                if isinstance(utd, UptodateCalculator):
+                    utd.setup(self, tasks_dict)
+                # 3) call it and get result
                 uptodate_result = utd(*args, **utd_kwargs)
+            # parameter is a value
             else:
                 uptodate_result = utd
 
@@ -360,8 +372,7 @@ class DependencyBase(object):
                 return 'run'
 
         # no dependencies means it is never up to date.
-        if ((not task.file_dep) and (not task.result_dep)
-            and (not checked_uptodate)):
+        if not (task.file_dep or checked_uptodate):
             return 'run'
 
         # if target file is not there, task is not up to date
@@ -370,7 +381,7 @@ class DependencyBase(object):
                 task.dep_changed = list(task.file_dep)
                 return 'run'
 
-        # check for modified dependencies
+        # check for modified file_dep
         changed = []
         status = 'up-to-date' # initial assumption
         for dep in tuple(task.file_dep):
@@ -382,13 +393,6 @@ class DependencyBase(object):
                 changed.append(dep)
                 status = 'run'
 
-        for dep in tuple(task.result_dep):
-            result = self._get(dep, "result:")
-            if ((result is None) or
-                (self._get(task.name, "task:" + dep) != result)):
-                status = 'run'
-                break
-
         task.dep_changed = changed #FIXME create a separate function for this
         return status
 
@@ -403,6 +407,22 @@ class DbmDependency(DependencyBase):
     def __init__(self, name):
         DependencyBase.__init__(self, DbmDB(name))
 
+
+class UptodateCalculator(object):
+    """Base class for 'uptodate' that need access to all tasks
+    """
+    def __init__(self):
+        self.get_val = None # Dependency._get
+        self.tasks_dict = None # dict with all tasks
+
+    def setup(self, dep_manager, tasks_dict):
+        """@param"""
+        self.get_val = dep_manager._get
+        self.tasks_dict = tasks_dict
+
+
+
+
 import platform
 python_version = platform.python_version().split('.')
 if python_version[0] == '2' and python_version[1] == '5': # pragma: no cover
