@@ -35,7 +35,7 @@ class Runner(object):
         @param always_execute: (bool) execute even if up-to-date or ignored
         @param verbosity: (int) 0,1,2 see Task.execute
         """
-        self.dependency_manager = Dependency(dependency_file)
+        self.dep_manager = Dependency(dependency_file)
         self.reporter = reporter
         self.continue_ = continue_
         self.always_execute = always_execute
@@ -53,7 +53,7 @@ class Runner(object):
         its execution is not successful.
         """
         assert isinstance(catched_excp, CatchedException)
-        self.dependency_manager.remove_success(task)
+        self.dep_manager.remove_success(task)
         self.reporter.add_failure(task, catched_excp)
         # only return FAILURE if no errors happened.
         if isinstance(catched_excp, TaskFailed):
@@ -72,16 +72,16 @@ class Runner(object):
             # if key_name is None get the whole dict
             if key_name is None:
                 if not tasks_dict[task_id].has_subtask:
-                    arg_value = self.dependency_manager.get_values(task_id)
+                    arg_value = self.dep_manager.get_values(task_id)
                 # if specified task is a group task pass values from all
                 # sub-tasks as string
                 else:
                     arg_value = []
                     for sub_id in tasks_dict[task_id].task_dep:
                         arg_value.append(
-                            self.dependency_manager.get_values(sub_id))
+                            self.dep_manager.get_values(sub_id))
             else:
-                arg_value = self.dependency_manager.get_value(task_id, key_name)
+                arg_value = self.dep_manager.get_value(task_id, key_name)
             task.options[arg] = arg_value
 
 
@@ -102,7 +102,7 @@ class Runner(object):
 
             # check if task is up-to-date
             try:
-                task.run_status = self.dependency_manager.get_status(task)
+                task.run_status = self.dep_manager.get_status(task, tasks_dict)
             except Exception, exception:
                 msg = "ERROR: Task '%s' checking dependencies" % task.name
                 dep_error = DependencyError(msg, exception)
@@ -113,7 +113,7 @@ class Runner(object):
                 # if task is up-to-date skip it
                 if task.run_status == 'up-to-date':
                     self.reporter.skip_uptodate(task)
-                    task.values = self.dependency_manager.get_values(task.name)
+                    task.values = self.dep_manager.get_values(task.name)
                     return False
                 # check if task should be ignored (user controlled)
                 if task.run_status == 'ignore':
@@ -155,7 +155,8 @@ class Runner(object):
         task.run_status = "done"
         # save execution successful
         if catched_excp is None:
-            self.dependency_manager.save_success(task)
+            task.save_extra_values()
+            self.dep_manager.save_success(task)
             self.reporter.add_success(task)
         # task error
         else:
@@ -170,11 +171,20 @@ class Runner(object):
 
         @param task_control: L{TaskControl}
         """
-        for task in task_control.task_dispatcher():
+        gen = task_control.task_dispatcher()
+        task = None
+        while True:
             if self._stop_running:
                 break
+
+            try:
+                task = gen.send(task)
+            except StopIteration:
+                break
+
             if not self.select_task(task, task_control.tasks):
                 continue
+
             catched_excp = self.execute_task(task)
             self.process_task_result(task, catched_excp)
 
@@ -194,7 +204,7 @@ class Runner(object):
     def finish(self):
         """finish running tasks"""
         # flush update dependencies
-        self.dependency_manager.close()
+        self.dep_manager.close()
         self.teardown()
 
         # report final results
@@ -217,9 +227,27 @@ class Hold(object):
     """Sentinel class: No task ready to be executed"""
     pass
 
+
 class MRunner(Runner):
     """MultiProcessing Runner """
 
+    # cant use staticmethod because of bug in py.test
+    # https://bitbucket.org/hpk42/pytest/issue/156
+    @classmethod
+    def available(cls):
+        """check if multiprocessing module is available"""
+        # see: https://bitbucket.org/schettino72/doit/issue/17
+        #      http://bugs.python.org/issue3770
+        # not available on BSD systens
+        try:
+            import multiprocessing.synchronize
+            multiprocessing # pyflakes
+        except ImportError: # pragma: no cover
+            return False
+        else:
+            return True
+
+
     class MReporter(object):
         """send reported messages to master process
 
@@ -255,7 +283,7 @@ class MRunner(Runner):
         self.tasks = None    # dict of task instances by name
         self.result_q = None
 
-    def get_next_task(self):
+    def get_next_task(self, completed):
         """get next task to be dispatched to sub-process
 
         On MP needs to check if the dependencies finished its execution
@@ -265,16 +293,17 @@ class MRunner(Runner):
         """
         if self._stop_running:
             return None # gentle stop
+        task = completed
         while True:
             # get next task from controller
             try:
-                task = self.task_gen.next()
+                task = self.task_gen.send(task)
                 if not isinstance(task, Task):
                     self.free_proc += 1
                     return Hold()
             # no more tasks from controller...
             except StopIteration:
-                # ... terminate one sub process
+                # ... terminate one sub process if no other task waiting
                 return None
 
             if self.select_task(task, self.tasks):
@@ -295,7 +324,7 @@ class MRunner(Runner):
         """
         proc_list = []
         for _ in xrange(self.num_process):
-            next_task = self.get_next_task()
+            next_task = self.get_next_task(None)
             if next_task is None:
                 break # do not start more processes than tasks
             task_q.put(next_task)
@@ -330,9 +359,14 @@ class MRunner(Runner):
             elif 'exit' in result:
                 raise result['exit'](result['exception'])
             else:
+                # success set values taken from subprocess result
                 catched_excp = None
                 task.result = result['result']
                 task.values = result['values']
+                for action, output in zip(task.actions, result['out']):
+                    action.out = output
+                for action, output in zip(task.actions, result['err']):
+                    action.err = output
 
             # completed one task, dispatch next one
             self.process_task_result(task, catched_excp)
@@ -341,8 +375,10 @@ class MRunner(Runner):
             free_proc = self.free_proc + 1
             self.free_proc = 0
             # tries to get as many tasks as free process
+            completed = task
             for _ in range(free_proc):
-                next_task = self.get_next_task()
+                next_task = self.get_next_task(completed)
+                completed = None
                 if next_task is None:
                     proc_count -= 1
                 task_q.put(next_task)
@@ -395,6 +431,8 @@ class MRunner(Runner):
                 if t_result is None:
                     result['result'] = task.result
                     result['values'] = task.values
+                    result['out'] = [a.out for a in task.actions]
+                    result['err'] = [a.err for a in task.actions]
                 else:
                     result['failure'] = t_result
                 result_q.put(result)
