@@ -3,6 +3,7 @@
 import types
 import os
 import copy
+import sys
 
 from . import cmdparse
 from .exceptions import CatchedException, InvalidTask
@@ -20,13 +21,14 @@ class Task(object):
     @ivar task_dep: (list - string)
     @ivar wild_dep: (list - string) task dependency using wildcard *
     @ivar file_dep: (set - string)
-    @ivar result_dep: (list -string)
     @ivar calc_dep: (set - string) reference to a task
-    @ivar calc_dep_stack: (list - string) unprocessed calc_dep
     @ivar dep_changed (list - string): list of file-dependencies that changed
           (are not up_to_date). this must be set before
     @ivar uptodate: (list - bool/None) use bool/computed value instead of
                                        checking dependencies
+    @ivar value_savers (list - callables) that return dicts to be added to
+                           task values. Always executed on main process.
+                           To be used by `uptodate` implementations.
     @ivar setup_tasks (list - string): references to task-names
     @ivar is_subtask: (bool) indicate this task is a subtask
     @ivar has_subtask: (bool) indicate this task has subtasks
@@ -87,8 +89,6 @@ class Task(object):
         self.check_attr(name, 'actions', actions, self.valid_attr['actions'])
         self.check_attr(name, 'file_dep', file_dep, self.valid_attr['file_dep'])
         self.check_attr(name, 'task_dep', task_dep, self.valid_attr['task_dep'])
-        self.check_attr(name, 'result_dep', result_dep,
-                        self.valid_attr['result_dep'])
         self.check_attr(name, 'uptodate', uptodate, self.valid_attr['uptodate'])
         self.check_attr(name, 'calc_dep', calc_dep, self.valid_attr['calc_dep'])
         self.check_attr(name, 'targets', targets, self.valid_attr['targets'])
@@ -105,9 +105,33 @@ class Task(object):
         self.name = name
         self.taskcmd = cmdparse.TaskOption(name, params, None, None)
         self.options = None
-        self.getargs = getargs
         self.setup_tasks = list(setup)
-        self._init_deps(file_dep, task_dep, result_dep, calc_dep)
+
+        # actions
+        self._action_instances = None
+        if actions is None:
+            self._actions = []
+        else:
+            self._actions = list(actions[:])
+
+        self._init_deps(file_dep, task_dep, calc_dep)
+
+        # DEPRECATED on 0.17 to be removed on 0.18
+        if result_dep: # pragma: no cover
+            from doit.tools import result_dep as check_result
+            uptodate = list(uptodate)
+            msg = ('DEPRECATION WARNING: Task "%s" is using "result_dep", '
+                   'use doit.tools.result_dep as "uptodate" instead.\n')
+            sys.__stderr__.write(msg % self.name)
+            for _result_dep in result_dep:
+                uptodate.append(check_result(_result_dep))
+
+        self.value_savers = []
+        self.uptodate = self._init_uptodate(uptodate) if uptodate else []
+
+        self.getargs = getargs
+        if self.getargs:
+            self._init_getargs()
         self.targets = targets
         self.is_subtask = is_subtask
         self.has_subtask = has_subtask
@@ -116,16 +140,6 @@ class Task(object):
         self.verbosity = verbosity
         self.custom_title = title
 
-        # actions
-        self._action_instances = None
-        if actions is None:
-            self._actions = []
-        else:
-            self._actions = actions[:]
-
-        # uptodate
-        self.uptodate = self._init_uptodate(uptodate) if uptodate else []
-
         # clean
         if clean is True:
             self._remove_targets = True
@@ -139,7 +153,7 @@ class Task(object):
         self.run_status = None
 
 
-    def _init_deps(self, file_dep, task_dep, result_dep, calc_dep):
+    def _init_deps(self, file_dep, task_dep, calc_dep):
         """init for dependency related attributes"""
         self.dep_changed = None
 
@@ -153,26 +167,21 @@ class Task(object):
         if task_dep:
             self._expand_task_dep(task_dep)
 
-        # result_dep
-        self.result_dep = []
-        if result_dep:
-            self._expand_result_dep(result_dep)
-
         # calc_dep
         self.calc_dep = set()
-        self.calc_dep_stack = []
         if calc_dep:
             self._expand_calc_dep(calc_dep)
 
-        # get args
-        if self.getargs:
-            self._init_getargs()
-
 
     def _init_uptodate(self, items):
         """wrap uptodate callables"""
         uptodate = []
         for item in items:
+            # configure task
+            if hasattr(item, 'configure_task'):
+                item.configure_task(self)
+
+            # check/append uptodate value to task
             if isinstance(item, bool) or item is None:
                 uptodate.append((item, None, None))
             elif hasattr(item, '__call__'):
@@ -207,19 +216,12 @@ class Task(object):
             else:
                 self.task_dep.append(dep)
 
-    def _expand_result_dep(self, result_dep):
-        """convert ressult_dep input into restul_dep and task_dep"""
-        self.result_dep.extend(result_dep)
-        # result_dep are also task_dep
-        self.task_dep.extend(result_dep)
-
 
     def _expand_calc_dep(self, calc_dep):
         """calc_dep input"""
         for dep in calc_dep:
             if dep not in self.calc_dep:
                 self.calc_dep.add(dep)
-                self.calc_dep_stack.append(dep)
 
 
     def _extend_uptodate(self, uptodate):
@@ -230,7 +232,6 @@ class Task(object):
     # FIXME should support setup also
     _expand_map = {'task_dep': _expand_task_dep,
                    'file_dep': _expand_file_dep,
-                   'result_dep': _expand_result_dep,
                    'calc_dep': _expand_calc_dep,
                    'uptodate': _extend_uptodate,
                    }
@@ -253,9 +254,15 @@ class Task(object):
         """task getargs attribute define implicit task dependencies"""
         self._init_options()
         for arg_name, desc in self.getargs.iteritems():
-            # value can a string task_id.key_name (deprecated)...
+
+            # DEPRECATED on 0.17, to be removed on 0.18
+            # value can be a string task_id.key_name
             if isinstance(desc, basestring):
-                # TODO raise deprecation warning
+                msg = ('DEPRECATION WARNING: Task "%s" "getargs" string values '
+                       '<task_id.key_name> is deprecated. '
+                       'Use a tuple (<task_id>, <key_name>) instead.\n')
+                sys.__stderr__.write(msg % self.name)
+
                 parts = desc.rsplit('.', 1)
                 if len(parts) != 2:
                     msg = ("Taskid '%s' - Invalid format for getargs of '%s'.\n" %
@@ -263,6 +270,8 @@ class Task(object):
                            "Should be <taskid>.<key-name> got '%s'\n" % desc)
                     raise InvalidTask(msg)
                 self.getargs[arg_name] = parts
+            # END - DEPRECATION
+
             # ... or tuple (task_id, key_name)
             else:
                 parts = desc
@@ -321,13 +330,23 @@ class Task(object):
         return self._action_instances
 
 
+    # deprecated on 0.17 to be removed on 0.18
     def insert_action(self, call_ref):
         """insert an action to execute before all other actions
 
         @param call_ref: callable or (callable, args, kwargs)
         This is part of interface to be used by 'uptodate' callables
         """
-        self._actions.insert(0, call_ref)
+        msg = ('DEPRECATION WARNING: Task "%s" insert_action is deprecated, '
+               'use "value_savers" instead.\n')
+        sys.__stderr__.write(msg % self.name)
+        self.value_savers.append(call_ref)
+    # end deprecation
+
+    def save_extra_values(self):
+        """run value_savers updating self.values"""
+        for value_saver in self.value_savers:
+            self.values.update(value_saver())
 
 
     def _get_out_err(self, out, err, verbosity):
@@ -432,6 +451,8 @@ class Task(object):
         del to_pickle['clean_actions']
         del to_pickle['teardown']
         del to_pickle['custom_title']
+        del to_pickle['value_savers']
+        del to_pickle['uptodate']
         return to_pickle
 
     def __eq__(self, other):
@@ -447,6 +468,7 @@ class Task(object):
         inst.name = self.name
         inst.targets = self.targets[:]
         inst.uptodate = self.uptodate[:]
+        inst.value_savers = self.value_savers[:]
         inst.is_subtask = self.is_subtask
         inst.has_subtask = self.has_subtask
         inst.result = self.result
@@ -466,9 +488,7 @@ class Task(object):
         inst.file_dep = copy.copy(self.file_dep)
         inst.task_dep = self.task_dep[:]
         inst.wild_dep = self.wild_dep[:]
-        inst.result_dep = self.result_dep[:]
         inst.calc_dep = copy.copy(self.calc_dep)
-        inst.calc_dep_stack = self.calc_dep_stack[:]
         inst.doc = self.doc
         inst.run_status = self.run_status
         return inst
