@@ -4,12 +4,14 @@ import threading
 import time
 
 import pytest
+from mock import Mock
 
 from doit.dependency import Dependency
 from doit.task import Task
 from doit.exceptions import InvalidCommand
 from doit import cmds
 from doit import reporter
+from doit import runner
 from tests.test_runner import FakeReporter
 from tests.conftest import remove_db
 
@@ -150,7 +152,6 @@ class TestCmdForget(object):
         cmds.doit_forget(depfile.name, tasks, output, [])
         got = output.getvalue().split("\n")[:-1]
         assert ["forgeting all tasks"] == got, repr(output.getvalue())
-#        assert False
         dep = Dependency(depfile.name)
         for task in tasks:
             assert None == dep._get(task.name, "dep")
@@ -206,6 +207,14 @@ class TestCmdRun(object):
         got = output.getvalue().split("\n")[:-1]
         assert [".  t1", ".  t2", ".  g1.a", ".  g1.b", ".  t3"] == got
 
+    def testMP_not_available(self, depfile, monkeypatch):
+        # make sure MRunner wont be used
+        monkeypatch.setattr(runner.MRunner, "available",
+                            Mock(return_value=False))
+        monkeypatch.setattr(runner.MRunner, "__init__", 'not available')
+        output = StringIO.StringIO()
+        cmds.doit_run(depfile.name, tasks_sample(), output, num_process=3)
+
     def testProcessRunMP(self, depfile):
         output = StringIO.StringIO()
         result = cmds.doit_run(depfile.name, tasks_sample(), output, num_process=1)
@@ -268,31 +277,46 @@ class TestCmdClean(object):
             self.cleaned = []
             def myclean(name):
                 self.cleaned.append(name)
-            self.tasks = [Task("t1", None, task_dep=['t2'],
-                               clean=[(myclean,('t1',))]),
-                          Task("t2", None, clean=[(myclean,('t2',))]),]
+            self.tasks = [
+                Task("t1", None, task_dep=['t2'], clean=[(myclean,('t1',))]),
+                Task("t2", None, clean=[(myclean,('t2',))]),
+                Task("t3", None, task_dep=['t3:a'], has_subtask=True,
+                     clean=[(myclean,('t3',))]),
+                Task("t3:a", None, clean=[(myclean,('t3:a',))]),
+                ]
         return request.cached_setup(
             setup=create_tasks,
             scope="function")
 
     def test_clean_all(self, tasks):
         output = StringIO.StringIO()
-        cmds.doit_clean(self.tasks, output, False, False, [])
-        assert ['t1','t2'] == self.cleaned
+        cmds.doit_clean(self.tasks, output, False, False, True, 'xxx', 'xxx')
+        assert ['t1','t2', 't3:a', 't3'] == self.cleaned
+
+    def test_clean_default(self, tasks):
+        output = StringIO.StringIO()
+        cmds.doit_clean(self.tasks, output, False, False, False, ['t1'], [])
+        # default enable --clean-dep by default
+        assert ['t2', 't1'] == self.cleaned
 
     def test_clean_selected(self, tasks):
         output = StringIO.StringIO()
-        cmds.doit_clean(self.tasks, output, False, False, ['t2'])
+        cmds.doit_clean(self.tasks, output, False, False, False, 'xxx', ['t2'])
         assert ['t2'] == self.cleaned
 
     def test_clean_taskdep(self, tasks):
         output = StringIO.StringIO()
-        cmds.doit_clean(self.tasks, output, False, True, ['t1'])
+        cmds.doit_clean(self.tasks, output, False, True, False, 'xxx', ['t1'])
         assert ['t2', 't1'] == self.cleaned
 
+    def test_clean_subtasks(self, tasks):
+        output = StringIO.StringIO()
+        cmds.doit_clean(self.tasks, output, False, False, False, 'xxx', ['t3'])
+        assert ['t3:a', 't3'] == self.cleaned
+
     def test_clean_taskdep_once(self, tasks):
         output = StringIO.StringIO()
-        cmds.doit_clean(self.tasks, output, False, True, ['t1', 't2'])
+        cmds.doit_clean(self.tasks, output, False, True, False, 'xxx', ['t1', 't2'])
         assert ['t2', 't1'] == self.cleaned
 
 
@@ -373,7 +397,7 @@ class TestCmdAuto(object):
         assert ["f1"] == w1_files
         # with calc_dep
         w2_tasks, w2_files = cmds._auto_watch([t1, t2], ["t2"])
-        assert ["t1", "t2"] == w2_tasks
+        assert sorted(["t1", "t2"]) == sorted(w2_tasks)
         assert ["f1", "f2"] == w2_files
 
 
@@ -429,7 +453,7 @@ class TestCmdAuto(object):
         fd.write("mod2")
         fd.close()
 
-        sleep_factor = 0.1 # ensure execution is over before start a new one
+        sleep_factor = 0.4 # ensure execution is over before start a new one
         time.sleep(sleep_factor)
         # write in another watched file ========> .  t2
         fd = open(file2, 'w')
