@@ -92,7 +92,7 @@ class TestRunner_SelectTask(object):
     def test_upToDate(self, reporter, depfile):
         t1 = Task("taskX", [(my_print, ["out a"] )], file_dep=[__file__])
         my_runner = runner.Runner(depfile.name, reporter)
-        my_runner.dependency_manager.save_success(t1)
+        my_runner.dep_manager.save_success(t1)
         assert False == my_runner.select_task(t1, {})
         assert ('start', t1) == reporter.log.pop(0)
         assert ('up-to-date', t1) == reporter.log.pop(0)
@@ -101,7 +101,7 @@ class TestRunner_SelectTask(object):
     def test_ignore(self, reporter, depfile):
         t1 = Task("taskX", [(my_print, ["out a"] )])
         my_runner = runner.Runner(depfile.name, reporter)
-        my_runner.dependency_manager.ignore(t1)
+        my_runner.dep_manager.ignore(t1)
         assert False == my_runner.select_task(t1, {})
         assert ('start', t1) == reporter.log.pop(0)
         assert ('ignore', t1) == reporter.log.pop(0)
@@ -110,7 +110,7 @@ class TestRunner_SelectTask(object):
     def test_alwaysExecute(self, reporter, depfile):
         t1 = Task("taskX", [(my_print, ["out a"] )])
         my_runner = runner.Runner(depfile.name, reporter, always_execute=True)
-        my_runner.dependency_manager.ignore(t1)
+        my_runner.dep_manager.ignore(t1)
         assert True == my_runner.select_task(t1, {})
         assert ('start', t1) == reporter.log.pop(0)
         assert not reporter.log
@@ -163,7 +163,7 @@ class TestRunner_SelectTask(object):
         # invalid getargs. Exception wil be raised and task will fail
         def check_x(my_x): return True
         t1 = Task('t1', [lambda :True])
-        t2 = Task('t2', [(check_x,)], getargs={'my_x':'t1.x'})
+        t2 = Task('t2', [(check_x,)], getargs={'my_x':('t1','x')})
         my_runner = runner.Runner(depfile.name, reporter)
 
         # t2 gives chance for setup tasks to be executed
@@ -258,12 +258,15 @@ class TestTask_Teardown(object):
 
 class TestTask_RunAll(object):
     def test_reporter_runtime_error(self, reporter, depfile):
-        t1 = Task('t1', [], setup=['make_invalid'])
+        t1 = Task('t1', [], calc_dep=['t2'])
+        t2 = Task('t2', [lambda: {'file_dep':[1]}])
         my_runner = runner.Runner(depfile.name, reporter)
-        tc = TaskControl([t1])
+        tc = TaskControl([t1, t2])
         tc.process(None)
         my_runner.run_all(tc)
-        assert ('start', t1) == reporter.log.pop(0)
+        assert ('start', t2) == reporter.log.pop(0)
+        assert ('execute', t2) == reporter.log.pop(0)
+        assert ('success', t2) == reporter.log.pop(0)
         assert ('runtime_error',) == reporter.log.pop(0)
         assert not reporter.log
 
@@ -271,7 +274,11 @@ class TestTask_RunAll(object):
 # run tests in both single process runner and multi-process runner
 def pytest_generate_tests(metafunc):
     if TestRunner_run_tasks == metafunc.cls:
-        for RunnerClass in (runner.Runner, runner.MRunner):
+        runners = [runner.Runner]
+        if runner.MRunner.available():
+            # TODO: test should be added and skipped!
+            runners.append(runner.MRunner)
+        for RunnerClass in (runners):
             metafunc.addcall(id=RunnerClass.__name__,
                              funcargs=dict(RunnerClass=RunnerClass))
 
@@ -333,6 +340,28 @@ class TestRunner_run_tasks(object):
         assert ('execute', tasks[1]) == reporter.log.pop(0)
         assert ('success', tasks[1]) == reporter.log.pop(0)
 
+    # test result, value, out, err are saved into task
+    def test_result(self, reporter, RunnerClass, depfile):
+        def my_action():
+            import sys
+            sys.stdout.write('out here')
+            sys.stderr.write('err here')
+            return {'bb': 5}
+        task = Task("taskY", [my_action] )
+        my_runner = RunnerClass(depfile.name, reporter)
+        tc = TaskControl([task])
+        tc.process(None)
+        assert None == task.result
+        assert {} == task.values
+        assert [None] == [a.out for a in task.actions]
+        assert [None] == [a.err for a in task.actions]
+        my_runner.run_tasks(tc)
+        assert runner.SUCCESS == my_runner.finish()
+        assert {'bb': 5} == task.result
+        assert {'bb': 5} == task.values
+        assert ['out here'] == [a.out for a in task.actions]
+        assert ['err here'] == [a.err for a in task.actions]
+
     # whenever a task fails remaining task are not executed
     def test_failureOutput(self, reporter, RunnerClass, depfile):
         tasks = [Task("taskX", [_fail]),
@@ -387,50 +416,6 @@ class TestRunner_run_tasks(object):
         d = Dependency(depfile.name)
         assert d._get("taskX", os.path.abspath(depPath))
 
-    def test_resultDependency(self, reporter, RunnerClass, depfile):
-        t1 = Task("t1", [(ok,)])
-        t2 = Task("t2", [(ok,)], result_dep=['t1'])
-        my_runner = RunnerClass(depfile.name, reporter)
-        tc = TaskControl([t1, t2])
-        tc.process(None)
-        my_runner.run_tasks(tc)
-        my_runner.finish()
-        assert ('start', t1) == reporter.log.pop(0)
-        assert ('execute', t1) == reporter.log.pop(0)
-        assert ('success', t1) == reporter.log.pop(0)
-        assert ('start', t2) == reporter.log.pop(0)
-        assert ('execute', t2) == reporter.log.pop(0)
-        assert ('success', t2) == reporter.log.pop(0)
-
-        # again
-        t1 = Task("t1", [(ok,)])
-        t2 = Task("t2", [(ok,)], result_dep=['t1'])
-        my_runner2 = RunnerClass(depfile.name, reporter)
-        tc2 = TaskControl([t1, t2])
-        tc2.process(None)
-        my_runner2.run_tasks(tc2)
-        my_runner2.finish()
-        assert ('start', t1) == reporter.log.pop(0)
-        assert ('execute', t1) == reporter.log.pop(0)
-        assert ('success', t1) == reporter.log.pop(0)
-        assert ('start', t2) == reporter.log.pop(0)
-        assert ('up-to-date', t2) == reporter.log.pop(0)
-
-        # change t1, t2 executed again
-        t1B = Task("t1", [(ok2,)])
-        t2 = Task("t2", [(ok,)], result_dep=['t1'])
-        my_runner3 = RunnerClass(depfile.name, reporter)
-        tc3 = TaskControl([t1B, t2])
-        tc3.process(None)
-        my_runner3.run_tasks(tc3)
-        my_runner3.finish()
-        assert ('start', t1B) == reporter.log.pop(0)
-        assert ('execute', t1B) == reporter.log.pop(0)
-        assert ('success', t1B) == reporter.log.pop(0)
-        assert ('start', t2) == reporter.log.pop(0)
-        assert ('execute', t2) == reporter.log.pop(0)
-        assert ('success', t2) == reporter.log.pop(0)
-
 
     def test_continue(self, reporter, RunnerClass, depfile):
         tasks = [Task("task1", [(_fail,)] ),
@@ -501,6 +486,8 @@ class TestMReporter(object):
         mp_reporter = runner.MRunner.MReporter(fake_runner, reporter)
         assert hasattr(mp_reporter, 'add_success')
         assert not hasattr(mp_reporter, 'no_existent_method')
+# python2.5 dont have class decorators
+pytest.mark.skipif('not runner.MRunner.available()')(TestMReporter)
 
 
 class TestMRunner_get_next_task(object):
@@ -512,9 +499,9 @@ class TestMRunner_get_next_task(object):
         tc.process(None)
         run = runner.MRunner(depfile.name, reporter)
         run._run_tasks_init(tc)
-        assert t1 == run.get_next_task()
-        assert t2 == run.get_next_task()
-        assert None == run.get_next_task()
+        assert t1 == run.get_next_task(None)
+        assert t2 == run.get_next_task(None)
+        assert None == run.get_next_task(None)
 
     def test_stop_running(self, reporter, depfile):
         t1 = Task('t1', [])
@@ -523,42 +510,27 @@ class TestMRunner_get_next_task(object):
         tc.process(None)
         run = runner.MRunner(depfile.name, reporter)
         run._run_tasks_init(tc)
-        assert t1 == run.get_next_task()
+        assert t1 == run.get_next_task(None)
         run._stop_running = True
-        assert None == run.get_next_task()
+        assert None == run.get_next_task(None)
 
     def test_waiting(self, reporter, depfile):
         t1 = Task('t1', [])
-        t2a = Task('t2A', [], task_dep=('t1',))
-        t2b = Task('t2B', [], setup=('t1',))
-        t3 = Task('t3', [], setup=('t2B',))
-        tc = TaskControl([t1, t2a, t2b, t3])
-        tc.process(None)
+        t2 = Task('t2', [], setup=('t1',))
+        tc = TaskControl([t1, t2])
+        tc.process(['t2'])
         run = runner.MRunner(depfile.name, reporter)
         run._run_tasks_init(tc)
 
-        # first start task t1
-        assert t1 == run.get_next_task()
-        t1.run_status = 'run'
-
+        # first start task 1
+        assert t1 == run.get_next_task(None)
         # hold until t1 is done
-        assert isinstance(run.get_next_task(), runner.Hold)
-        assert 1 == run.free_proc
-        t2b.run_status = 'run'
-        t3.run_status = 'run'
-
-        # t2a can run
+        assert isinstance(run.get_next_task(None), runner.Hold)
+        assert isinstance(run.get_next_task(None), runner.Hold)
         t1.run_status = 'done'
-        assert t2a == run.get_next_task()
-        # t2b second time run
-        assert t2b == run.get_next_task()
 
-        # hold until t3 can run
-        assert isinstance(run.get_next_task(), runner.Hold)
-        t2b.run_status = 'done'
-        assert t3 == run.get_next_task()
-
-        assert None == run.get_next_task()
+        assert t2 == run.get_next_task(t1)
+        assert None == run.get_next_task(t2)
 
 
     def test_waiting_controller(self, reporter, depfile):
@@ -570,12 +542,14 @@ class TestMRunner_get_next_task(object):
         run._run_tasks_init(tc)
 
         # first task ok
-        assert t1 == run.get_next_task()
+        assert t1 == run.get_next_task(None)
 
         # hold until t1 finishes
         assert 0 == run.free_proc
-        assert isinstance(run.get_next_task(), runner.Hold)
+        assert isinstance(run.get_next_task(None), runner.Hold)
         assert 1 == run.free_proc
+# python2.5 dont have class decorators
+pytest.mark.skipif('not runner.MRunner.available()')(TestMRunner_get_next_task)
 
 
 class TestMRunner_start_process(object):
@@ -635,6 +609,9 @@ class TestMRunner_start_process(object):
         assert 2 == len(proc_list)
         assert t1.name == task_q.get().name
         assert isinstance(task_q.get(), runner.Hold)
+# python2.5 dont have class decorators
+pytest.mark.skipif('not runner.MRunner.available()')(TestMRunner_start_process)
+
 
 class TestMRunner_execute_task(object):
     def test_hold(self, reporter, depfile):
@@ -647,4 +624,6 @@ class TestMRunner_execute_task(object):
         run.finish()
         # nothing was done
         assert result_q.empty() # pragma: no cover (coverage bug?)
+# python2.5 dont have class decorators
+pytest.mark.skipif('not runner.MRunner.available()')(TestMRunner_execute_task)
 
