@@ -71,12 +71,20 @@ class TestTaskInit(object):
         assert t.run_status is None
 
 
-class TestTaskInsertAction(object):
+class TestTaskValueSavers(object):
     def test_insert_action(self):
         t = task.Task("Task X", ["taskcmd"])
         def void(task, values, (my_arg1,)): pass
         t.insert_action((void, [1]))
-        assert 2 == len(t.actions)
+        assert 1 == len(t.actions)
+        assert 1 == len(t.value_savers)
+
+    def test_execute_value_savers(self):
+        t = task.Task("Task X", ["taskcmd"])
+        t.value_savers.append(lambda: {'v1':1})
+        t.save_extra_values()
+        assert 1 == t.values['v1']
+
 
 
 class TestTaskUpToDate(object):
@@ -107,6 +115,16 @@ class TestTaskUpToDate(object):
                       uptodate=[(custom_check, [123], {'xxx':'yyy'})])
         assert t.uptodate[0] == (custom_check, [123], {'xxx':'yyy'})
 
+    def test_object_with_configure(self):
+        class Check(object):
+            def __call__(self): return True
+            def configure_task(self, task):
+                task.task_dep.append('y1')
+        check = Check()
+        t = task.Task("Task X", ["taskcmd"], uptodate=[check])
+        assert (check, [], {}) == t.uptodate[0]
+        assert ['y1'] == t.task_dep
+
     def test_invalid(self):
         pytest.raises(task.InvalidTask,
                       task.Task, "Task X", ["taskcmd"], uptodate=[{'x':'y'}])
@@ -135,36 +153,27 @@ class TestTaskDeps(object):
         assert ["123"] == my_task.task_dep
         assert ["4*56"] == my_task.wild_dep
 
-    def test_expand_result(self):
-        my_task = task.Task("Task X", ["taskcmd"], result_dep=["123"])
-        assert ["123"] == my_task.result_dep
-        assert ["123"] == my_task.task_dep
-
     def test_calc_dep(self):
         my_task = task.Task("Task X", ["taskcmd"], calc_dep=["123"])
         assert set(["123"]) == my_task.calc_dep
-        assert ["123"] == my_task.calc_dep_stack
 
     def test_update_deps(self):
         my_task = task.Task("Task X", ["taskcmd"], file_dep=["fileX"],
-                            calc_dep=["calcX"], result_dep=["resultX"],
-                            uptodate=[None])
+                            calc_dep=["calcX"], uptodate=[None])
         my_task.update_deps({'file_dep': ['fileY'],
                              'task_dep': ['taskY'],
                              'calc_dep': ['calcX', 'calcY'],
-                             'result_dep': ['resultY'],
                              'uptodate': [True],
                              })
         assert set(['fileX', 'fileY']) == my_task.file_dep
-        assert ['resultX', 'taskY', 'resultY'] == my_task.task_dep
+        assert ['taskY'] == my_task.task_dep
         assert set(['calcX', 'calcY']) == my_task.calc_dep
-        assert ['resultX', 'resultY'] == my_task.result_dep
         assert [(None, None, None), (True, None, None)] == my_task.uptodate
 
 
 class TestTask_Getargs(object):
     def test_ok(self):
-        getargs = {'x' : 't1.x', 'y': 't2.z'}
+        getargs = {'x' : ('t1','x'), 'y': ('t2','z')}
         t = task.Task('t3', None, getargs=getargs)
         assert 't1' in t.setup_tasks
         assert 't2' in t.setup_tasks
@@ -180,6 +189,7 @@ class TestTask_Getargs(object):
                               't3', None, getargs=getargs)
 
     def test_many_dots(self):
+        # DEPRECATED
         getargs = {'x': 't2:file.ext.x'}
         t = task.Task('t1', None, getargs=getargs)
         assert 't2:file.ext' in t.setup_tasks
