@@ -38,12 +38,80 @@ class TestRunOnce(object):
     def test_run(self):
         t = task.Task("TaskX", None, uptodate=[tools.run_once])
         assert False == tools.run_once(t, t.values)
-        t.execute()
+        t.save_extra_values()
         assert True == tools.run_once(t, t.values)
 
 
-class TestConfigChanged(object):\
+class TestResultDep(object):
+    def test_single(self, depfile):
+        dep_manager = depfile
 
+        tasks = {'t1': task.Task("t1", None, uptodate=[tools.result_dep('t2')]),
+                 't2': task.Task("t2", None),
+                 }
+        # _config_task was executed and t2 added as task_dep
+        assert ['t2'] == tasks['t1'].task_dep
+
+        # first t2 result
+        tasks['t2'].result = 'yes'
+        dep_manager.save_success(tasks['t2'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)  # first time
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+        # t2 result changed
+        tasks['t2'].result = '222'
+        dep_manager.save_success(tasks['t2'])
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+
+    def test_group(self, depfile):
+        dep_manager = depfile
+
+        tasks = {'t1': task.Task("t1", None, uptodate=[tools.result_dep('t2')]),
+                 't2': task.Task("t2", None, task_dep=['t2:a', 't2:b'],
+                                 has_subtask=True),
+                 't2:a': task.Task("t2:a", None),
+                 't2:b': task.Task("t2:b", None),
+                 }
+        # _config_task was executed and t2 added as task_dep
+        assert ['t2'] == tasks['t1'].task_dep
+
+        # first t2 result
+        tasks['t2:a'].result = 'yes1'
+        dep_manager.save_success(tasks['t2:a'])
+        tasks['t2:b'].result = 'yes2'
+        dep_manager.save_success(tasks['t2:b'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)  # first time
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+        # t2 result changed
+        tasks['t2:a'].result = '222'
+        dep_manager.save_success(tasks['t2:a'])
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'run' == dep_manager.get_status(tasks['t1'], tasks)
+
+        tasks['t1'].save_extra_values()
+        dep_manager.save_success(tasks['t1'])
+        assert 'up-to-date' == dep_manager.get_status(tasks['t1'], tasks)
+
+
+
+class TestConfigChanged(object):
     def test_invalid_type(self):
         class NotValid(object):pass
         uptodate = tools.config_changed(NotValid())
@@ -55,7 +123,7 @@ class TestConfigChanged(object):\
         t1 = task.Task("TaskX", None, uptodate=[ua])
         assert False == ua(t1, t1.values)
         assert False == ub(t1, t1.values)
-        t1.execute()
+        t1.save_extra_values()
         assert True == ua(t1, t1.values)
         assert False == ub(t1, t1.values)
 
@@ -65,7 +133,7 @@ class TestConfigChanged(object):\
         t1 = task.Task("TaskX", None, uptodate=[ua])
         assert False == ua(t1, t1.values)
         assert False == ub(t1, t1.values)
-        t1.execute()
+        t1.save_extra_values()
         assert True == ua(t1, t1.values)
         assert False == ub(t1, t1.values)
 
@@ -75,7 +143,7 @@ class TestConfigChanged(object):\
         t1 = task.Task("TaskX", None, uptodate=[ua])
         assert False == ua(t1, t1.values)
         assert False == ub(t1, t1.values)
-        t1.execute()
+        t1.save_extra_values()
         assert True == ua(t1, t1.values)
         assert False == ub(t1, t1.values)
 
@@ -90,7 +158,7 @@ class TestTimeout(object):
         t = task.Task("TaskX", None, uptodate=[uptodate])
 
         assert False == uptodate(t, t.values)
-        t.execute()
+        t.save_extra_values()
         assert 100 == t.values['success-time']
 
         monkeypatch.setattr(tools.time_module, 'time', lambda: 103)
@@ -107,7 +175,7 @@ class TestTimeout(object):
         t = task.Task("TaskX", None, uptodate=[uptodate])
 
         assert False == uptodate(t, t.values)
-        t.execute()
+        t.save_extra_values()
         assert 10 == t.values['success-time']
 
         monkeypatch.setattr(tools.time_module, 'time', lambda: 100)
@@ -124,7 +192,7 @@ class TestTimeout(object):
         t = task.Task("TaskX", None, uptodate=[uptodate])
 
         assert False == uptodate(t, t.values)
-        t.execute()
+        t.save_extra_values()
         assert 10 == t.values['success-time']
 
         monkeypatch.setattr(tools.time_module, 'time', lambda: 3600 * 30)
@@ -178,7 +246,7 @@ class TestCheckTimestampUnchanged(object):
         assert False == check(t, t.values)
 
         # value just stored is equal to itself
-        t.execute()
+        t.save_extra_values()
         assert True == check(t, t.values)
 
         # stored timestamp less than current, up to date
